@@ -3,9 +3,8 @@
     Copyright (C) 1997,1998  Matt Kimball
 
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+    it under the terms of the GNU General Public License version 2 as 
+    published by the Free Software Foundation.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,6 +29,9 @@
 #include "select.h"
 #include "raw.h"
 #include "dns.h"
+#ifndef NO_IPINFO
+#include <asn.h>
+#endif
 
 extern int DisplayMode;
 
@@ -95,8 +97,12 @@ void display_open(void)
     break;
   case DisplayCurses:
     mtr_curses_open();  
+#ifndef NO_IPINFO
+    if (ipinfo_no >= 0)
+        asn_open();
+#endif
     break;
-  case DisplaySplit:            /* BL */
+  case DisplaySplit:
     split_open();
     break;
   case DisplayGTK:
@@ -106,7 +112,7 @@ void display_open(void)
 }
 
 
-void display_close(void)
+void display_close(time_t now)
 {
   switch(DisplayMode) {
   case DisplayReport:
@@ -119,12 +125,16 @@ void display_close(void)
     xml_close();
     break;
   case DisplayCSV:
-    csv_close();
+    csv_close(now);
     break;
   case DisplayCurses:
+#ifndef NO_IPINFO
+    if (ipinfo_no >= 0)
+        asn_close();
+#endif
     mtr_curses_close();
     break;
-  case DisplaySplit:            /* BL */
+  case DisplaySplit:
     split_close();
     break;
   case DisplayGTK:
@@ -142,7 +152,7 @@ void display_redraw(void)
     mtr_curses_redraw();
     break;
 
-  case DisplaySplit:            /* BL */
+  case DisplaySplit:
     split_redraw();
     break;
 
@@ -159,7 +169,7 @@ int display_keyaction(void)
   case DisplayCurses:
     return mtr_curses_keyaction();
 
-  case DisplaySplit:		/* BL */
+  case DisplaySplit:
     return split_keyaction();
 
   case DisplayGTK:
@@ -176,7 +186,7 @@ void display_rawping(int host, int msec)
   case DisplayTXT:
   case DisplayXML:
   case DisplayCSV:
-  case DisplaySplit:            /* BL */
+  case DisplaySplit:
   case DisplayCurses:
   case DisplayGTK:
     break;
@@ -194,7 +204,7 @@ void display_rawhost(int host, ip_t *ip_addr)
   case DisplayTXT:
   case DisplayXML:
   case DisplayCSV:
-  case DisplaySplit:            /* BL */
+  case DisplaySplit:
   case DisplayCurses:
   case DisplayGTK:
     break;
@@ -212,7 +222,7 @@ void display_loop(void)
   case DisplayTXT:
   case DisplayXML:
   case DisplayCSV:
-  case DisplaySplit:            /* BL */
+  case DisplaySplit:
   case DisplayCurses:
   case DisplayRaw:
     select_loop();
@@ -234,7 +244,7 @@ void display_clear(void)
   case DisplayTXT:
   case DisplayXML:
   case DisplayCSV:
-  case DisplaySplit:            /* BL */
+  case DisplaySplit:
   case DisplayRaw:
     break;
 
